@@ -25,6 +25,12 @@
 
 #include <errno.h>
 #include <string.h>
+#include <unistd.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <sys/ioctl.h>
+#include <net/ethernet.h>
+#include <net/if.h>
 
 #include "connman.h"
 
@@ -54,7 +60,6 @@ struct connman_device {
 	char *ident;
 	char *path;
 	char *devname;
-	int phyindex;
 	int index;
 	guint pending_timeout;
 
@@ -238,7 +243,7 @@ int __connman_device_disable(struct connman_device *device)
 
 	if (device->network) {
 		struct connman_service *service =
-			__connman_service_lookup_from_network(device->network);
+			connman_service_lookup_from_network(device->network);
 
 		if (service != NULL)
 			__connman_service_disconnect(service);
@@ -423,12 +428,10 @@ struct connman_device *connman_device_create(const char *node,
 	device->type = type;
 	device->name = g_strdup(type2description(device->type));
 
-	device->phyindex = -1;
-
 	device->networks = g_hash_table_new_full(g_str_hash, g_str_equal,
 						g_free, free_network);
 
-	device_list = g_slist_append(device_list, device);
+	device_list = g_slist_prepend(device_list, device);
 
 	return device;
 }
@@ -514,17 +517,6 @@ int connman_device_get_index(struct connman_device *device)
 	return device->index;
 }
 
-int __connman_device_get_phyindex(struct connman_device *device)
-{
-	return device->phyindex;
-}
-
-void __connman_device_set_phyindex(struct connman_device *device,
-							int phyindex)
-{
-	device->phyindex = phyindex;
-}
-
 /**
  * connman_device_set_interface:
  * @device: device structure
@@ -603,14 +595,17 @@ int connman_device_set_powered(struct connman_device *device,
 	connman_device_set_disconnected(device, FALSE);
 	device->scanning = FALSE;
 
-	if (device->driver && device->driver->scan_fast)
-		device->driver->scan_fast(device);
-	else if (device->driver && device->driver->scan)
-		device->driver->scan(device);
+	if (device->driver && device->driver->scan)
+		device->driver->scan(device, NULL, 0, NULL, NULL, NULL);
 
 	return 0;
 }
 
+connman_bool_t connman_device_get_powered(struct connman_device *device)
+{
+	return device->powered;
+}
+
 static int device_scan(struct connman_device *device)
 {
 	if (!device->driver || !device->driver->scan)
@@ -619,7 +614,7 @@ static int device_scan(struct connman_device *device)
 	if (device->powered == FALSE)
 		return -ENOLINK;
 
-	return device->driver->scan(device);
+	return device->driver->scan(device, NULL, 0, NULL, NULL, NULL);
 }
 
 int __connman_device_disconnect(struct connman_device *device)
@@ -671,7 +666,8 @@ static void mark_network_unavailable(gpointer key, gpointer value,
 {
 	struct connman_network *network = value;
 
-	if (connman_network_get_connected(network) == TRUE)
+	if (connman_network_get_connected(network) == TRUE ||
+			connman_network_get_connecting(network) == TRUE)
 		return;
 
 	connman_network_set_available(network, FALSE);
@@ -1132,16 +1128,189 @@ int __connman_device_request_hidden_scan(struct connman_device *device,
 	DBG("device %p", device);
 
 	if (device == NULL || device->driver == NULL ||
-			device->driver->scan_hidden == NULL)
+			device->driver->scan == NULL)
 		return -EINVAL;
 
 	if (device->scanning == TRUE)
 		return -EALREADY;
 
-	return device->driver->scan_hidden(device, ssid, ssid_len,
+	return device->driver->scan(device, ssid, ssid_len,
 					identity, passphrase, user_data);
 }
 
+static char *index2ident(int index, const char *prefix)
+{
+	struct ifreq ifr;
+	struct ether_addr eth;
+	char *str;
+	int sk, err, len;
+
+	if (index < 0)
+		return NULL;
+
+	sk = socket(PF_INET, SOCK_DGRAM | SOCK_CLOEXEC, 0);
+	if (sk < 0)
+		return NULL;
+
+	memset(&ifr, 0, sizeof(ifr));
+	ifr.ifr_ifindex = index;
+
+	err = ioctl(sk, SIOCGIFNAME, &ifr);
+
+	if (err == 0)
+		err = ioctl(sk, SIOCGIFHWADDR, &ifr);
+
+	close(sk);
+
+	if (err < 0)
+		return NULL;
+
+	len = prefix ? strlen(prefix) + 18 : 18;
+
+	str = g_malloc(len);
+	if (!str)
+		return NULL;
+
+	memcpy(&eth, &ifr.ifr_hwaddr.sa_data, sizeof(eth));
+	snprintf(str, len, "%s%02x%02x%02x%02x%02x%02x",
+						prefix ? prefix : "",
+						eth.ether_addr_octet[0],
+						eth.ether_addr_octet[1],
+						eth.ether_addr_octet[2],
+						eth.ether_addr_octet[3],
+						eth.ether_addr_octet[4],
+						eth.ether_addr_octet[5]);
+
+	return str;
+}
+
+static char *index2addr(int index)
+{
+	struct ifreq ifr;
+	struct ether_addr eth;
+	char *str;
+	int sk, err;
+
+	if (index < 0)
+		return NULL;
+
+	sk = socket(PF_INET, SOCK_DGRAM | SOCK_CLOEXEC, 0);
+	if (sk < 0)
+		return NULL;
+
+	memset(&ifr, 0, sizeof(ifr));
+	ifr.ifr_ifindex = index;
+
+	err = ioctl(sk, SIOCGIFNAME, &ifr);
+
+	if (err == 0)
+		err = ioctl(sk, SIOCGIFHWADDR, &ifr);
+
+	close(sk);
+
+	if (err < 0)
+		return NULL;
+
+	str = g_malloc(18);
+	if (!str)
+		return NULL;
+
+	memcpy(&eth, &ifr.ifr_hwaddr.sa_data, sizeof(eth));
+	snprintf(str, 18, "%02X:%02X:%02X:%02X:%02X:%02X",
+						eth.ether_addr_octet[0],
+						eth.ether_addr_octet[1],
+						eth.ether_addr_octet[2],
+						eth.ether_addr_octet[3],
+						eth.ether_addr_octet[4],
+						eth.ether_addr_octet[5]);
+
+	return str;
+}
+
+struct connman_device *connman_device_create_from_index(int index)
+{
+	enum connman_device_type type;
+	struct connman_device *device;
+	char *devname, *ident = NULL;
+	char *addr = NULL, *name = NULL;
+
+	if (index < 0)
+		return NULL;
+
+	devname = connman_inet_ifname(index);
+	if (devname == NULL)
+		return NULL;
+
+	if (__connman_device_isfiltered(devname) == TRUE) {
+		connman_info("Ignoring interface %s (filtered)", devname);
+		g_free(devname);
+		return NULL;
+	}
+
+	type = __connman_rtnl_get_device_type(index);
+
+	switch (type) {
+	case CONNMAN_DEVICE_TYPE_UNKNOWN:
+		connman_info("Ignoring interface %s (type unknown)", devname);
+		g_free(devname);
+		return NULL;
+	case CONNMAN_DEVICE_TYPE_ETHERNET:
+	case CONNMAN_DEVICE_TYPE_GADGET:
+	case CONNMAN_DEVICE_TYPE_WIFI:
+	case CONNMAN_DEVICE_TYPE_WIMAX:
+		name = index2ident(index, "");
+		addr = index2addr(index);
+		break;
+	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
+	case CONNMAN_DEVICE_TYPE_CELLULAR:
+	case CONNMAN_DEVICE_TYPE_GPS:
+	case CONNMAN_DEVICE_TYPE_VENDOR:
+		name = g_strdup(devname);
+		break;
+	}
+
+	device = connman_device_create(name, type);
+	if (device == NULL)
+		goto done;
+
+	switch (type) {
+	case CONNMAN_DEVICE_TYPE_UNKNOWN:
+	case CONNMAN_DEVICE_TYPE_VENDOR:
+	case CONNMAN_DEVICE_TYPE_GPS:
+		break;
+	case CONNMAN_DEVICE_TYPE_ETHERNET:
+	case CONNMAN_DEVICE_TYPE_GADGET:
+		ident = index2ident(index, NULL);
+		break;
+	case CONNMAN_DEVICE_TYPE_WIFI:
+	case CONNMAN_DEVICE_TYPE_WIMAX:
+		ident = index2ident(index, NULL);
+		break;
+	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
+		break;
+	case CONNMAN_DEVICE_TYPE_CELLULAR:
+		ident = index2ident(index, NULL);
+		break;
+	}
+
+	connman_device_set_index(device, index);
+	connman_device_set_interface(device, devname);
+
+	if (ident != NULL) {
+		connman_device_set_ident(device, ident);
+		g_free(ident);
+	}
+
+	connman_device_set_string(device, "Address", addr);
+
+done:
+	g_free(devname);
+	g_free(name);
+	g_free(addr);
+
+	return device;
+}
+
 connman_bool_t __connman_device_isfiltered(const char *devname)
 {
 	char **pattern;
